﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using Devart.Data.Oracle;
using Devart.Data.Oracle.Entity;

namespace OracleFullTextSearch {

  class Program {

    static void Main(string[] args) {

      //using (var ctx = new MyContext()) {
      //  ctx.Database.ExecuteSqlCommand("DELETE FROM docs");
      //  ctx.Database.ExecuteSqlCommand("DELETE FROM auction");
      //  ctx.Database.ExecuteSqlCommand("DELETE FROM queries");
      //}

      var monitor = new OracleMonitor() { IsActive = true };

      UseContextIndex();
      UseCtxCatIndex();
      UseCtxRuleIndex();

      Console.ReadKey();
    }

    private static void UseContextIndex() {

      using (var ctx = new MyContext()) {

        // Load documents into table DOCS
        ctx.Docs.Add(new Doc() { Id = 1, Text = "<HTML>California is a state in the US.</HTML>" });
        ctx.Docs.Add(new Doc() { Id = 2, Text = "<HTML>Paris is a city in France.</HTML>" });
        ctx.Docs.Add(new Doc() { Id = 3, Text = "<HTML>France is in Europe.</HTML>" });
        ctx.SaveChanges();

        // Synchronize the index
        ctx.CtxDdl.SyncIndex("idx_docs", "2M");

        // Querying your table with CONTAINS
        // Напишем LINQ to Entities аналог для SQL запроса:
        // SELECT SCORE(1), id, text FROM docs WHERE CONTAINS(text, 'France', 1) > 0
        var firstQuery = ctx.Docs
          .Where(doc => OracleTextFunctions.Contains(doc.Text, "France", 1) > 0)
          .Select(doc => new {
            Score = OracleTextFunctions.Score(1),
            Doc = doc
          });
        var firstQueryResults = firstQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("Querying your table with CONTAINS");
        Console.WriteLine();
        Console.WriteLine("1) CONTAINS('France')");
        Console.WriteLine();
        Console.WriteLine("{0,-5} | {1,-5} | {2}", "Score", "Id", "Title");
        Console.WriteLine(new string('-', 40));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-5} | {1,-5} | {2}", item.Score, item.Doc.Id, item.Doc.Text);

        // Add some rows to the DOCS table
        ctx.Docs.Add(new Doc() { Id = 4, Text = "<HTML>Los Angeles is a city in California.</HTML>" });
        ctx.Docs.Add(new Doc() { Id = 5, Text = "<HTML>Mexico City is big.</HTML>" });
        ctx.SaveChanges();

        // Напишем LINQ to Entities аналог для SQL запроса:
        //   SELECT SCORE(1), id, text FROM docs WHERE CONTAINS(text, 'city', 1) > 0
        var secondQuery = ctx.Docs
          .Where(doc => OracleTextFunctions.Contains(doc.Text, "city", 1) > 0)
          .Select(doc => new {
            Score = OracleTextFunctions.Score(1),
            Doc = doc
          });
        var secondQueryResults = secondQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("2) CONTAINS('city')");
        Console.WriteLine();
        Console.WriteLine("{0,-5} | {1,-5} | {2}", "Score", "Id", "Title");
        Console.WriteLine(new string('-', 40));
        foreach (var item in secondQuery)
          Console.WriteLine("{0,-5} | {1,-5} | {2}", item.Score, item.Doc.Id, item.Doc.Text);

        // Synchronize the index
        ctx.Database.ExecuteSqlCommand("BEGIN CTX_DDL.SYNC_INDEX('idx_docs', '2M'); END;");

        // Выполним второй запрос заново
        var secondQueryNewResults = secondQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("3) CONTAINS('city') after index synchronization");
        Console.WriteLine();
        Console.WriteLine("{0,-5} | {1,-5} | {2}", "Score", "Id", "Title");
        Console.WriteLine(new string('-', 40));
        foreach (var item in secondQuery)
          Console.WriteLine("{0,-5} | {1,-5} | {2}", item.Score, item.Doc.Id, item.Doc.Text);
      }

      PressAnyKey();
    }

    private static void UseCtxCatIndex() {

      using (var ctx = new MyContext()) {

        // Pupulate AUCTION table
        ctx.Auctions.Add(new Auction { Id = 1, Title = "NIKON CAMERA", CategoryId = 1, Price = 400, BidClose = new DateTime(2002, 10, 24) });
        ctx.Auctions.Add(new Auction { Id = 2, Title = "OLYMPUS CAMERA", CategoryId = 1, Price = 300, BidClose = new DateTime(2002, 10, 25) });
        ctx.Auctions.Add(new Auction { Id = 3, Title = "PENTAX CAMERA", CategoryId = 1, Price = 200, BidClose = new DateTime(2002, 10, 26) });
        ctx.Auctions.Add(new Auction { Id = 4, Title = "CANON CAMERA", CategoryId = 1, Price = 250, BidClose = new DateTime(2002, 10, 27) });
        ctx.SaveChanges();

        // Querying your table with CATSEARCH
        // Напишем LINQ to Entities аналог для SQL запроса:
        //   SELECT title, price FROM auction WHERE CATSEARCH(title, 'CAMERA', 'order by price') > 0;
        var firstQuery = ctx.Auctions
          .Where(a => OracleTextFunctions.Catsearch(a.Title, "CAMERA", "order by price") > 0)
          .Select(a => new { a.Title, a.Price });
        var firstQueryResults = firstQuery.ToList();

        // Output
        Console.WriteLine();
        Console.WriteLine("Querying your table with CATSEARCH");
        Console.WriteLine();
        Console.WriteLine("1) order by price");
        Console.WriteLine();
        Console.WriteLine("{0,-20} | {1}", "Title", "Price");
        Console.WriteLine(new string('-', 30));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-20} | {1}", item.Title, item.Price);

        // Напишем LINQ to Entities аналог для SQL запроса:
        //   SELECT title, price FROM auction WHERE CATSEARCH(title, 'CAMERA', 'price <= 300') > 0;
        var secondQuery = ctx.Auctions
          .Where(a => OracleTextFunctions.Catsearch(a.Title, "CAMERA", "price <= 300") > 0)
          .Select(a => new { a.Title, a.Price });
        var secondQueryResults = secondQuery.ToList();

        // Output
        Console.WriteLine();
        Console.WriteLine("2) price <= 300");
        Console.WriteLine();
        Console.WriteLine("{0,-20} | {1}", "Title", "Price");
        Console.WriteLine(new string('-', 30));
        foreach (var item in secondQueryResults)
          Console.WriteLine("{0,-20} | {1}", item.Title, item.Price);

        // Add some rows to the DOCS table
        ctx.Auctions.Add(new Auction { Id = 5, Title = "FUJI CAMERA", CategoryId = 1, Price = 350, BidClose = new DateTime(2002, 10, 28) });
        ctx.Auctions.Add(new Auction { Id = 6, Title = "SONY CAMERA", CategoryId = 1, Price = 310, BidClose = new DateTime(2002, 10, 28) });
        ctx.SaveChanges();

        // Выполним первый запрос заново
        var firstQueryNewResults = firstQuery.ToList();

        // Output
        Console.WriteLine();
        Console.WriteLine("3) order by price (после добавления новых записей)");
        Console.WriteLine();
        Console.WriteLine("{0,-20} | {1}", "Title", "Price");
        Console.WriteLine(new string('-', 30));
        foreach (var item in firstQueryNewResults)
          Console.WriteLine("{0,-20} | {1}", item.Title, item.Price);
      }

      PressAnyKey();
    }

    private static void UseCtxRuleIndex() {

      using (var ctx = new MyContext()) {

        // Load documents into QUERIES table
        ctx.Queries.Add(new Query() { Id = 1, QueryString = "oracle" });
        ctx.Queries.Add(new Query() { Id = 2, QueryString = "larry or ellison" });
        ctx.Queries.Add(new Query() { Id = 3, QueryString = "oracle and text" });
        ctx.Queries.Add(new Query() { Id = 4, QueryString = "market share" });
        ctx.SaveChanges();

        // Synchronize the index
        ctx.Database.ExecuteSqlCommand("BEGIN CTX_DDL.SYNC_INDEX('queryx', '2M'); END;");

        // Classify with MATCHES
        // Напишем LINQ to Entities аналог для SQL запроса:
        //   SELECT query_id,query_string FROM queries WHERE MATCHES(query_string,
        //     'Oracle announced that its market share in databases increased over the last year.') > 0;
        var firstQuery = ctx.Queries
          .Where(q => OracleTextFunctions.Matches(q.QueryString, "Oracle announced that its market share in databases increased over the last year.") > 0);
        var firstQueryResults = firstQuery.ToList();

        // Output:
        Console.WriteLine();
        Console.WriteLine("Classify with MATCHES");
        Console.WriteLine();
        Console.WriteLine("{0,-10} | {1}", "QueryId", "QueryString");
        Console.WriteLine(new string('-', 30));
        foreach (var item in firstQueryResults)
          Console.WriteLine("{0,-10} | {1}", item.Id, item.QueryString);
      }
    }

    private static void PressAnyKey() {

      Console.WriteLine();
      Console.Write("Press any key for continue...");
      Console.ReadKey();
      Console.WriteLine();
      Console.WriteLine();
    }

  }

}
